@@ -40,7 +40,7 @@ class VersionedMetadata:
     def __init__(self, *, version=None, name=None, lib_deps=None,
             other_lib_deps=None, exes=None, files=None, others=None, dll=True,
             qml=False, qml_names=None, translations=None,
-            excluded_plugins=None):
+            excluded_plugins=None, lgpl=True):
         """ Initialise the versioned bindings. """
 
         self._version = version
@@ -56,6 +56,8 @@ class VersionedMetadata:
         self._translations = () if translations is None else translations
         self._excluded_plugins = excluded_plugins
 
+        self.lgpl = lgpl
+
     def bundle(self, name, target_qt_dir, qt_dir, arch, qt_version,
             ignore_missing):
         """ Bundle part of Qt as defined by the meta-data. """
@@ -327,7 +329,9 @@ class VersionedMetadata:
             return 'libQt{}{}.so.{}'.format(qt_major, name[2:], qt_major)
 
         if arch == 'macos':
-            return '{}.framework/Versions/{}/{}'.format(name, qt_major, name)
+            framework = '5' if qt_major == 5 else 'A'
+
+            return '{}.framework/Versions/{}/{}'.format(name, framework, name)
 
         if arch == 'win':
             return 'Qt{}{}.dll'.format(qt_major, name[2:])
